#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################


import simplejson as json

from django.contrib.auth.models import User
from django.conf import settings
from django.core.urlresolvers import reverse

from beat.web.dataformats.models import DataFormat

import beat.core.algorithm

from ..models import Algorithm

from .core import AlgorithmsAPIBase


class AlgorithmsListRetrieval(AlgorithmsAPIBase):

    def test_only_public_and_usable_algorithms_for_anonymous_user(self):
        url = reverse('api_algorithms:all')
        response = self.client.get(url)
        content = self.checkResponse(response, 200, content_type='application/json')

        self.assertTrue(isinstance(content, list))
        self.assertEqual(len(content), 2)

        algorithm = filter(lambda x: x['name'] == 'jackdoe/public_for_all/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['accessibility'], 'public')
        self.assertFalse(algorithm.has_key('is_owner'))

        algorithm = filter(lambda x: x['name'] == 'jackdoe/usable_by_all/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['accessibility'], 'confidential')
        self.assertFalse(algorithm.has_key('is_owner'))

    def test_all_accessible_algorithms(self):
        self.client.login(username='johndoe', password='1234')

        url = reverse('api_algorithms:all')
        response = self.client.get(url)

        content = self.checkResponse(response, 200, content_type='application/json')

        self.assertTrue(isinstance(content, list))
        self.assertEqual(len(content), 9)

        algorithm = filter(lambda x: x['name'] == 'jackdoe/usable_by_one_user/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertEqual(algorithm['version'], 1)
        self.assertEqual(algorithm['previous_version'], None)
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['is_owner'], False)
        self.assertEqual(algorithm['accessibility'], 'confidential')

        algorithm = filter(lambda x: x['name'] == 'jackdoe/usable_by_one_team/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertEqual(algorithm['version'], 1)
        self.assertEqual(algorithm['previous_version'], None)
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['is_owner'], False)
        self.assertEqual(algorithm['accessibility'], 'confidential')

        algorithm = filter(lambda x: x['name'] == 'jackdoe/usable_by_teams/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertEqual(algorithm['version'], 1)
        self.assertEqual(algorithm['previous_version'], None)
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['is_owner'], False)
        self.assertEqual(algorithm['accessibility'], 'confidential')

        algorithm = filter(lambda x: x['name'] == 'jackdoe/public_for_all/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertEqual(algorithm['version'], 1)
        self.assertEqual(algorithm['previous_version'], None)
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['is_owner'], False)
        self.assertEqual(algorithm['accessibility'], 'public')

        algorithm = filter(lambda x: x['name'] == 'jackdoe/usable_by_all/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertEqual(algorithm['version'], 1)
        self.assertEqual(algorithm['previous_version'], None)
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['is_owner'], False)
        self.assertEqual(algorithm['accessibility'], 'confidential')

        algorithm = filter(lambda x: x['name'] == 'jackdoe/public_for_one_user/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertEqual(algorithm['version'], 1)
        self.assertEqual(algorithm['previous_version'], None)
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['is_owner'], False)
        self.assertEqual(algorithm['accessibility'], 'confidential')

        algorithm = filter(lambda x: x['name'] == 'jackdoe/public_for_one_team/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertEqual(algorithm['version'], 1)
        self.assertEqual(algorithm['previous_version'], None)
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['is_owner'], False)
        self.assertEqual(algorithm['accessibility'], 'confidential')

        algorithm = filter(lambda x: x['name'] == 'jackdoe/public_for_teams/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertEqual(algorithm['version'], 1)
        self.assertEqual(algorithm['previous_version'], None)
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['is_owner'], False)
        self.assertEqual(algorithm['accessibility'], 'confidential')

        algorithm = filter(lambda x: x['name'] == 'johndoe/forked_algo/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertEqual(algorithm['version'], 1)
        self.assertTrue(algorithm['previous_version'] is None)
        self.assertEqual(algorithm['fork_of'], 'jackdoe/public_for_all/1')
        self.assertEqual(algorithm['is_owner'], True)
        self.assertEqual(algorithm['accessibility'], 'private')


    def test_own_algorithms(self):
        self.client.login(username='jackdoe', password='1234')

        url = reverse('api_algorithms:list_create', args=['jackdoe'])
        response = self.client.get(url)

        content = self.checkResponse(response, 200, content_type='application/json')

        self.assertTrue(isinstance(content, list))
        self.assertEqual(len(content), 9)

        self.assertEqual(len(filter(lambda x: x['name'] == 'jackdoe/usable_by_one_user/1', content)), 1)
        self.assertEqual(len(filter(lambda x: x['name'] == 'jackdoe/usable_by_one_team/1', content)), 1)
        self.assertEqual(len(filter(lambda x: x['name'] == 'jackdoe/usable_by_teams/1', content)), 1)
        self.assertEqual(len(filter(lambda x: x['name'] == 'jackdoe/usable_by_all/1', content)), 1)
        self.assertEqual(len(filter(lambda x: x['name'] == 'jackdoe/public_for_all/1', content)), 1)
        self.assertEqual(len(filter(lambda x: x['name'] == 'jackdoe/public_for_one_user/1', content)), 1)
        self.assertEqual(len(filter(lambda x: x['name'] == 'jackdoe/public_for_one_team/1', content)), 1)
        self.assertEqual(len(filter(lambda x: x['name'] == 'jackdoe/public_for_teams/1', content)), 1)
        self.assertEqual(len(filter(lambda x: x['name'] == 'jackdoe/personal/1', content)), 1)


        algorithm = filter(lambda x: x['name'] == 'jackdoe/usable_by_one_user/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertTrue(algorithm['fork_of'] is None)
        # self.assertEqual(len(algorithm['inputs']), 2)
        # self.assertEqual(len(algorithm['outputs']), 1)

        # input = algorithm['inputs'][0]
        # self.assertEqual(input['name'], 'a')
        # self.assertEqual(input['dataformat'], 'johndoe/single_integer/1')

        # input = algorithm['inputs'][1]
        # self.assertEqual(input['name'], 'b')
        # self.assertEqual(input['dataformat'], 'johndoe/single_integer/1')

        # output = algorithm['outputs'][0]
        # self.assertEqual(output['name'], 'sum')
        # self.assertEqual(output['dataformat'], 'johndoe/single_integer/1')


        algorithm = filter(lambda x: x['name'] == 'jackdoe/public_for_all/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertTrue(algorithm['fork_of'] is None)
        # self.assertEqual(len(algorithm['inputs']), 2)
        # self.assertEqual(len(algorithm['outputs']), 1)

        # input = algorithm['inputs'][0]
        # self.assertEqual(input['name'], 'a')
        # self.assertEqual(input['dataformat'], 'johndoe/single_integer/1')

        # input = algorithm['inputs'][1]
        # self.assertEqual(input['name'], 'b')
        # self.assertEqual(input['dataformat'], 'johndoe/single_integer/1')

        # output = algorithm['outputs'][0]
        # self.assertEqual(output['name'], 'sum')
        # self.assertEqual(output['dataformat'], 'johndoe/single_integer/1')


    def test_other_user_algorithms(self):
        self.client.login(username='johndoe', password='1234')

        url = reverse('api_algorithms:list_create', args=['jackdoe'])
        response = self.client.get(url)

        content = self.checkResponse(response, 200, content_type='application/json')

        self.assertTrue(isinstance(content, list))
        self.assertEqual(len(content), 8)

        algorithm = filter(lambda x: x['name'] == 'jackdoe/usable_by_one_user/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['accessibility'], 'confidential')
        self.assertFalse(algorithm['opensource'])

        algorithm = filter(lambda x: x['name'] == 'jackdoe/usable_by_one_team/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['accessibility'], 'confidential')
        self.assertFalse(algorithm['opensource'])

        algorithm = filter(lambda x: x['name'] == 'jackdoe/usable_by_teams/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['accessibility'], 'confidential')
        self.assertFalse(algorithm['opensource'])

        algorithm = filter(lambda x: x['name'] == 'jackdoe/usable_by_all/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['accessibility'], 'confidential')
        self.assertFalse(algorithm['opensource'])

        algorithm = filter(lambda x: x['name'] == 'jackdoe/public_for_one_user/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['accessibility'], 'confidential')
        self.assertTrue(algorithm['opensource'])

        algorithm = filter(lambda x: x['name'] == 'jackdoe/public_for_one_team/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['accessibility'], 'confidential')
        self.assertTrue(algorithm['opensource'])

        algorithm = filter(lambda x: x['name'] == 'jackdoe/public_for_teams/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['accessibility'], 'confidential')
        self.assertTrue(algorithm['opensource'])

        algorithm = filter(lambda x: x['name'] == 'jackdoe/public_for_all/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['accessibility'], 'public')
        self.assertTrue(algorithm['opensource'])


    def test_user_algorithms_for_anonymous_user(self):
        url = reverse('api_algorithms:list_create', args=['jackdoe'])
        response = self.client.get(url)

        content = self.checkResponse(response, 200, content_type='application/json')

        self.assertTrue(isinstance(content, list))
        self.assertEqual(len(content), 1)

        algorithm = filter(lambda x: x['name'] == 'jackdoe/public_for_all/1', content)[0]
        self.assertEqual(algorithm['short_description'], '')
        self.assertTrue(algorithm['fork_of'] is None)
        self.assertEqual(algorithm['accessibility'], 'public')
        self.assertTrue(algorithm['opensource'])


class AlgorithmsNameCheck(AlgorithmsAPIBase):

    def setUp(self):
        super(AlgorithmsNameCheck, self).setUp()

        self.url = reverse('api_algorithms:check_name')

        user = User.objects.get(username='jackdoe')
        (version4, errors) = Algorithm.objects.create_algorithm(
            author=user,
            name='algorithm-4',
            short_description='short description 4'
            )
        assert version4, errors


    def test_no_check_for_anonymous_user(self):
        response = self.client.post(self.url)
        self.checkResponse(response, 403)


    def test_bad_request_for_post_method_without_needed_data(self):
        self.client.login(username='jackdoe', password='1234')
        response = self.client.post(self.url,
                                    json.dumps({}),
                                    content_type='application/json')
        self.checkResponse(response, 400)


    def test_not_used_valid_name(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'valid-name1',
                'private': True
            }), content_type='application/json')

        content = self.checkResponse(response, 200, content_type='application/json')

        self.assertTrue(isinstance(content, dict))
        self.assertEqual(content['name'], 'valid-name1')
        self.assertFalse(content['used'])


    def test_not_used_invalid_name(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'invalid name_1 #due-to{}%&é',
                'private': True
            }), content_type='application/json')

        content = self.checkResponse(response, 200, content_type='application/json')

        self.assertTrue(isinstance(content, dict))
        self.assertEqual(content['name'], 'invalid-name_1--due-to-----')
        self.assertFalse(content['used'])


    def test_used_valid_name(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'usable_by_one_user',
                'private': True
            }), content_type='application/json')

        content = self.checkResponse(response, 200, content_type='application/json')

        self.assertTrue(isinstance(content, dict))
        self.assertEqual(content['name'], 'usable_by_one_user')
        self.assertTrue(content['used'])


    def test_used_invalid_name(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'algorithm 4',
                'private': True
            }), content_type='application/json')

        content = self.checkResponse(response, 200, content_type='application/json')

        self.assertTrue(isinstance(content, dict))
        self.assertEqual(content['name'], 'algorithm-4')
        self.assertTrue(content['used'])



class AlgorithmCreation(AlgorithmsAPIBase):

    def setUp(self):
        super(AlgorithmCreation, self).setUp()

        self.url = reverse('api_algorithms:list_create', args=['jackdoe'])

        user = User.objects.get(username='jackdoe')
        (version4, errors) = Algorithm.objects.create_algorithm(
            author=user,
            name='algorithm-4',
            short_description='short description 4'
            )
        assert version4, errors


    def checkAlgorithm(self, algorithm, declaration=None, code=None):

        default = beat.core.algorithm.Algorithm(settings.PREFIX, data=None)
        declaration = declaration or json.dumps(default.data, indent=2)
        code = code or default.code

        storage = beat.core.algorithm.Storage(settings.PREFIX,
                algorithm.fullname())
        assert storage.exists

        in_storage = json.loads(storage.json.load())
        expected = json.loads(declaration)

        assert in_storage == expected, "There are differences on the " \
                "declaration (beyond white spaces): %r != %r" %  \
                (in_storage, expected)

        # set storage language so it can find the code
        storage.language = in_storage['language']

        in_storage = storage.code.try_load()
        expected = code

        assert in_storage == expected, "There are differences on the " \
                "code: %r != %r" % (in_storage, expected)


    def test_no_creation_for_anonymous_user(self):
        response = self.client.post(self.url)
        self.checkResponse(response, 403)


    def test_no_creation_for_other_user(self):
        self.client.login(username='jackdoe', password='1234')
        url = reverse('api_algorithms:list_create', args=['johndoe'])
        response = self.client.post(url)
        self.checkResponse(response, 403)


    def test_bad_request_for_post_method_without_needed_data(self):
        self.client.login(username='jackdoe', password='1234')
        response = self.client.post(self.url,
                                    json.dumps({}),
                                    content_type='application/json')
        self.checkResponse(response, 400)


    def test_not_used_valid_name_no_code(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'valid-name1',
                'description': 'blah'
            }), content_type='application/json')

        url = reverse('api_algorithms:object', args=['jackdoe', 'valid-name1', 1])
        content = self.checkResponse(response, 201,
                                     content_type='application/json',
                                     location=url)

        self.assertTrue(isinstance(content, dict))
        self.assertEqual(content['name'], 'valid-name1')
        self.assertTrue(content['url'].endswith(url))

        try:
            algorithm = Algorithm.objects.get(author__username='jackdoe', name='valid-name1')
        except:
            self.assertTrue(False)

        self.assertEqual(algorithm.short_description, '')
        self.assertEqual(algorithm.description, 'blah')
        self.checkAlgorithm(algorithm)


    def test_not_used_valid_name__no_description_no_code(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'valid-name1'
            }), content_type='application/json')

        url = reverse('api_algorithms:object', args=['jackdoe', 'valid-name1', 1])
        content = self.checkResponse(response, 201,
                                     content_type='application/json',
                                     location=url)

        self.assertTrue(isinstance(content, dict))
        self.assertEqual(content['name'], 'valid-name1')
        self.assertTrue(content['url'].endswith(url))

        try:
            algorithm = Algorithm.objects.get(author__username='jackdoe', name='valid-name1')
        except:
            self.assertTrue(False)

        self.assertEqual(algorithm.short_description, '')
        self.checkAlgorithm(algorithm)


    def test_not_used_invalid_name_no_code(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'invalid name1',
                'description': 'blah'
            }), content_type='application/json')

        url = reverse('api_algorithms:object', args=['jackdoe', 'invalid-name1', 1])
        content = self.checkResponse(response, 201,
                                     content_type='application/json',
                                     location=url)

        self.assertTrue(isinstance(content, dict))
        self.assertEqual(content['name'], 'invalid-name1')
        self.assertTrue(content['url'].endswith(url))

        try:
            algorithm = Algorithm.objects.get(author__username='jackdoe', name='invalid-name1')
        except:
            self.assertTrue(False)

        self.assertEqual(algorithm.short_description, '')
        self.assertEqual(algorithm.description, 'blah')
        self.checkAlgorithm(algorithm)


    def test_not_used_invalid_name__no_description_no_code(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'invalid name1'
            }), content_type='application/json')

        url = reverse('api_algorithms:object', args=['jackdoe', 'invalid-name1', 1])
        content = self.checkResponse(response, 201,
                                     content_type='application/json',
                                     location=url)

        self.assertTrue(isinstance(content, dict))
        self.assertEqual(content['name'], 'invalid-name1')
        self.assertTrue(content['url'].endswith(url))

        try:
            algorithm = Algorithm.objects.get(author__username='jackdoe', name='invalid-name1')
        except:
            self.assertTrue(False)

        self.assertEqual(algorithm.short_description, '')
        self.checkAlgorithm(algorithm)


    def test_not_used_valid_name(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'valid-name1',
                'description': 'blah',
                'declaration':AlgorithmsAPIBase.DECLARATION,
                'code': AlgorithmsAPIBase.CODE
            }), content_type='application/json')

        url = reverse('api_algorithms:object', args=['jackdoe', 'valid-name1', 1])
        content = self.checkResponse(response, 201,
                                     content_type='application/json',
                                     location=url)

        self.assertTrue(isinstance(content, dict))
        self.assertEqual(content['name'], 'valid-name1')
        self.assertTrue(content['url'].endswith(url))

        try:
            algorithm = Algorithm.objects.get(author__username='jackdoe', name='valid-name1')
        except:
            self.assertTrue(False)

        self.assertEqual(algorithm.short_description, '')
        self.assertEqual(algorithm.description, 'blah')
        self.checkAlgorithm(algorithm, AlgorithmsAPIBase.DECLARATION,
                AlgorithmsAPIBase.CODE)


    def test_not_used_valid_name__no_description(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'valid-name1',
                'declaration': AlgorithmsAPIBase.DECLARATION,
                'code': AlgorithmsAPIBase.CODE,
            }), content_type='application/json')

        url = reverse('api_algorithms:object', args=['jackdoe', 'valid-name1', 1])
        content = self.checkResponse(response, 201,
                                     content_type='application/json',
                                     location=url)

        self.assertTrue(isinstance(content, dict))
        self.assertEqual(content['name'], 'valid-name1')
        self.assertTrue(content['url'].endswith(url))

        try:
            algorithm = Algorithm.objects.get(author__username='jackdoe', name='valid-name1')
        except:
            self.assertTrue(False)

        self.assertEqual(algorithm.short_description, '')
        self.checkAlgorithm(algorithm, AlgorithmsAPIBase.DECLARATION,
                AlgorithmsAPIBase.CODE)


    def test_not_used_invalid_name(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'invalid name1',
                'description': 'blah',
                'declaration': AlgorithmsAPIBase.DECLARATION,
                'code': AlgorithmsAPIBase.CODE,
            }), content_type='application/json')

        url = reverse('api_algorithms:object', args=['jackdoe', 'invalid-name1', 1])
        content = self.checkResponse(response, 201,
                                     content_type='application/json',
                                     location=url)

        self.assertTrue(isinstance(content, dict))
        self.assertEqual(content['name'], 'invalid-name1')
        self.assertTrue(content['url'].endswith(url))

        try:
            algorithm = Algorithm.objects.get(author__username='jackdoe', name='invalid-name1')
        except:
            self.assertTrue(False)

        self.assertEqual(algorithm.short_description, '')
        self.assertEqual(algorithm.description, 'blah')
        self.checkAlgorithm(algorithm, AlgorithmsAPIBase.DECLARATION,
                AlgorithmsAPIBase.CODE)


    def test_not_used_invalid_name__no_description(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'invalid name1',
                'declaration': AlgorithmsAPIBase.DECLARATION,
                'code': AlgorithmsAPIBase.CODE,
            }), content_type='application/json')

        url = reverse('api_algorithms:object', args=['jackdoe', 'invalid-name1', 1])
        content = self.checkResponse(response, 201,
                                     content_type='application/json',
                                     location=url)

        self.assertTrue(isinstance(content, dict))
        self.assertEqual(content['name'], 'invalid-name1')
        self.assertTrue(content['url'].endswith(url))

        try:
            algorithm = Algorithm.objects.get(author__username='jackdoe', name='invalid-name1')
        except:
            self.assertTrue(False)

        self.assertEqual(algorithm.short_description, '')
        self.checkAlgorithm(algorithm, AlgorithmsAPIBase.DECLARATION,
                AlgorithmsAPIBase.CODE)


    def test_used_valid_name_no_code(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'usable_by_one_user',
                'description': 'blah'
            }), content_type='application/json')

        self.checkResponse(response, 400, content_type='application/json')


    def test_used_valid_name__no_description_no_code(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'usable_by_one_user'
            }), content_type='application/json')

        self.checkResponse(response, 400, content_type='application/json')


    def test_used_invalid_name_no_code(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'algorithm 4',
                'description': 'blah'
            }), content_type='application/json')

        self.checkResponse(response, 400, content_type='application/json')


    def test_used_invalid_name__no_description_no_code(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'algorithm 4'
            }), content_type='application/json')

        self.checkResponse(response, 400, content_type='application/json')


    def test_used_valid_name(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'usable_by_one_user',
                'description': 'blah',
                'declaration': AlgorithmsAPIBase.DECLARATION,
                'code': AlgorithmsAPIBase.CODE,
            }), content_type='application/json')

        self.checkResponse(response, 400, content_type='application/json')


    def test_used_valid_name__no_description(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'usable_by_one_user',
                'declaration': AlgorithmsAPIBase.DECLARATION,
                'code': AlgorithmsAPIBase.CODE,
            }), content_type='application/json')

        self.checkResponse(response, 400, content_type='application/json')


    def test_used_invalid_name(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'algorithm 4',
                'description': 'blah',
                'declaration': AlgorithmsAPIBase.DECLARATION,
                'code': AlgorithmsAPIBase.CODE
            }), content_type='application/json')

        self.checkResponse(response, 400, content_type='application/json')


    def test_used_invalid_name__no_description(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'name': 'algorithm 4',
                'declaration': AlgorithmsAPIBase.DECLARATION,
                'code': AlgorithmsAPIBase.CODE,
            }), content_type='application/json')

        self.checkResponse(response, 400, content_type='application/json')


class AlgorithmUpdate(AlgorithmsAPIBase):
    def setUp(self):
        super(AlgorithmUpdate, self).setUp()

        self.url = reverse('api_algorithms:object', args=['jackdoe', 'personal', 1])

    def test_no_update_for_anonymous_user(self):
        response = self.client.put(self.url,
            json.dumps({
                'description': 'blah',
                'declaration': AlgorithmsAPIBase.DECLARATION,
                'code': AlgorithmsAPIBase.CODE,
                }), content_type='application/json')

        self.checkResponse(response, 403)


    def test_fail_to_update_with_invalid_username(self):
        self.client.login(username='jackdoe', password='1234')
        url = reverse('api_algorithms:object', args=['unknown', 'personal', 1])

        response = self.client.put(url,
            json.dumps({
                'description': 'blah',
                'declaration': AlgorithmsAPIBase.DECLARATION,
                'code': AlgorithmsAPIBase.CODE,
            }), content_type='application/json')

        self.checkResponse(response, 403)


    def test_fail_to_update_with_invalid_algorithm_name(self):
        self.client.login(username='jackdoe', password='1234')
        url = reverse('api_algorithms:object', args=['jackdoe', 'unknown', 1])

        response = self.client.put(url,
            json.dumps({
                'description': 'blah',
                'declaration': AlgorithmsAPIBase.DECLARATION,
                'code': AlgorithmsAPIBase.CODE,
            }), content_type='application/json')

        self.checkResponse(response, 404)


    def test_no_update_without_content(self):
        self.client.login(username='jackdoe', password='1234')
        response = self.client.put(self.url)
        self.checkResponse(response, 400)


    def test_successfull_update(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.put(self.url,
            json.dumps({
                'description': 'blah',
                'declaration': AlgorithmsAPIBase.UPDATE
            }), content_type='application/json')

        self.checkResponse(response, 204)

        algorithm = Algorithm.objects.get(author__username='jackdoe', name='personal', version=1)
        self.assertEqual(algorithm.description, 'blah')

        storage = beat.core.algorithm.Storage(settings.PREFIX,
                algorithm.fullname())
        storage.language = 'python'
        assert storage.exists()
        self.assertEqual(storage.json.load(), AlgorithmsAPIBase.UPDATE)


    def test_successfull_update_description_only(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.put(self.url,
            json.dumps({
                'description': 'blah',
            }), content_type='application/json')

        self.checkResponse(response, 204)

        algorithm = Algorithm.objects.get(author__username='jackdoe', name='personal', version=1)
        self.assertEqual(algorithm.description, 'blah')


    def test_successfull_update_code_only(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.put(self.url,
            json.dumps({
                'declaration': AlgorithmsAPIBase.UPDATE
            }), content_type='application/json')

        self.checkResponse(response, 204)

        algorithm = Algorithm.objects.get(author__username='jackdoe', name='personal', version=1)

        storage = beat.core.algorithm.Storage(settings.PREFIX,
                algorithm.fullname())
        storage.language = 'python'
        assert storage.exists()
        self.assertEqual(storage.json.load(), AlgorithmsAPIBase.UPDATE)

    def test_successfull_update_change_input_name(self):
        declaration = """{
  "language": "python",
  "splittable": false,
  "groups": [
    {
      "inputs": {
        "changed": { "type": "johndoe/single_integer/1" },
        "b": { "type": "johndoe/single_integer/1" }
      },
      "outputs": {
        "sum": { "type": "johndoe/single_integer/1" }
      }
    }
  ],
  "parameters": {
  }
}"""

        self.client.login(username='jackdoe', password='1234')

        response = self.client.put(self.url,
            json.dumps({
                'declaration': declaration,
            }), content_type='application/json')

        self.checkResponse(response, 204)

        algorithm = Algorithm.objects.get(author__username='jackdoe',
                                          name='personal', version=1)

        self.assertEqual(algorithm.endpoints.count(), 3)

        endpoint = algorithm.endpoints.filter(name='changed')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='b')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='sum')[0]
        self.assertFalse(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')


    def test_successfull_update_change_input_format(self):
        declaration = """{
  "language": "python",
  "splittable": false,
  "groups": [
    {
      "inputs": {
        "a": { "type": "jackdoe/single_float/1" },
        "b": { "type": "johndoe/single_integer/1" }
      },
      "outputs": {
        "sum": { "type": "johndoe/single_integer/1" }
      }
    }
  ],
  "parameters": {
  }
}"""

        (dataformat, errors) = DataFormat.objects.create_dataformat(
            author=User.objects.get(username='jackdoe'),
            name='single_float',
            )
        assert dataformat, errors

        self.client.login(username='jackdoe', password='1234')

        response = self.client.put(self.url,
            json.dumps({
                'declaration': declaration,
            }), content_type='application/json')

        self.checkResponse(response, 204)

        algorithm = Algorithm.objects.get(author__username='jackdoe',
                                          name='personal', version=1)

        self.assertEqual(algorithm.endpoints.count(), 3)

        endpoint = algorithm.endpoints.filter(name='a')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'jackdoe/single_float/1')

        endpoint = algorithm.endpoints.filter(name='b')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='sum')[0]
        self.assertFalse(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')


    def test_successfull_update_change_output_name(self):
        declaration = """{
  "language": "python",
  "splittable": false,
  "groups": [
    {
      "inputs": {
        "a": { "type": "johndoe/single_integer/1" },
        "b": { "type": "johndoe/single_integer/1" }
      },
      "outputs": {
        "changed": { "type": "johndoe/single_integer/1" }
      }
    }
  ],
  "parameters": {
  }
}"""
        code = """class Algorithm:

    def process(self, inputs, outputs):
        return True

"""

        self.client.login(username='jackdoe', password='1234')

        response = self.client.put(self.url,
            json.dumps({
                'declaration': declaration,
                'code': code,
            }), content_type='application/json')

        self.checkResponse(response, 204)

        algorithm = Algorithm.objects.get(author__username='jackdoe',
                                          name='personal', version=1)

        self.assertEqual(algorithm.endpoints.count(), 3)

        endpoint = algorithm.endpoints.filter(name='a')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='b')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='changed')[0]
        self.assertFalse(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')


    def test_successfull_update_change_output_format(self):
        declaration = """{
  "language": "python",
  "splittable": false,
  "groups": [
    {
      "inputs": {
        "a": { "type": "johndoe/single_integer/1" },
        "b": { "type": "johndoe/single_integer/1" }
      },
      "outputs": {
        "sum": { "type": "jackdoe/single_float/1" }
      }
    }
  ],
  "parameters": {
  }
}"""

        (dataformat, errors) = DataFormat.objects.create_dataformat(
            author=User.objects.get(username='jackdoe'),
            name='single_float',
            )
        assert dataformat, errors

        self.client.login(username='jackdoe', password='1234')

        response = self.client.put(self.url,
            json.dumps({
                'declaration': declaration
            }), content_type='application/json')

        self.checkResponse(response, 204)

        algorithm = Algorithm.objects.get(author__username='jackdoe',
                                          name='personal', version=1)

        self.assertEqual(algorithm.endpoints.count(), 3)

        endpoint = algorithm.endpoints.filter(name='a')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='b')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='sum')[0]
        self.assertFalse(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'jackdoe/single_float/1')


    def test_successfull_update_add_input(self):
        declaration = """{
  "language": "python",
  "splittable": false,
  "groups": [
    {
      "inputs": {
        "a": { "type": "johndoe/single_integer/1" },
        "b": { "type": "johndoe/single_integer/1" },
        "c": { "type": "johndoe/single_integer/1" }
      },
      "outputs": {
        "sum": { "type": "johndoe/single_integer/1" }
      }
    }
  ],
  "parameters": {
  }
}"""

        self.client.login(username='jackdoe', password='1234')

        response = self.client.put(self.url,
            json.dumps({
                'declaration': declaration,
            }), content_type='application/json')

        self.checkResponse(response, 204)

        algorithm = Algorithm.objects.get(author__username='jackdoe',
                                          name='personal', version=1)

        self.assertEqual(algorithm.endpoints.count(), 4)

        endpoint = algorithm.endpoints.filter(name='a')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='b')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='c')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='sum')[0]
        self.assertFalse(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')


    def test_successfull_update_remove_input(self):
        declaration = """{
  "language": "python",
  "splittable": false,
  "groups": [
    {
      "inputs": {
        "a": { "type": "johndoe/single_integer/1" }
      },
      "outputs": {
        "sum": { "type": "johndoe/single_integer/1" }
      }
    }
  ],
  "parameters": {
  }
}"""

        self.client.login(username='jackdoe', password='1234')

        response = self.client.put(self.url,
            json.dumps({
                'declaration': declaration,
            }), content_type='application/json')

        self.checkResponse(response, 204)

        algorithm = Algorithm.objects.get(author__username='jackdoe',
                                          name='personal', version=1)

        self.assertEqual(algorithm.endpoints.count(), 2)

        endpoint = algorithm.endpoints.filter(name='a')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='sum')[0]
        self.assertFalse(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')


    def test_successfull_update_add_output(self):
        declaration = """{
  "language": "python",
  "splittable": false,
  "groups": [
    {
      "inputs": {
        "a": { "type": "johndoe/single_integer/1" },
        "b": { "type": "johndoe/single_integer/1" }
      },
      "outputs": {
        "sum": { "type": "johndoe/single_integer/1" },
        "diff": { "type": "johndoe/single_integer/1" }
      }
    }
  ],
  "parameters": {
  }
}"""

        self.client.login(username='jackdoe', password='1234')

        response = self.client.put(self.url,
            json.dumps({
                'declaration': declaration,
            }), content_type='application/json')

        self.checkResponse(response, 204)

        algorithm = Algorithm.objects.get(author__username='jackdoe',
                                          name='personal', version=1)

        self.assertEqual(algorithm.endpoints.count(), 4)

        endpoint = algorithm.endpoints.filter(name='a')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='b')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='sum')[0]
        self.assertFalse(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='diff')[0]
        self.assertFalse(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')


    def test_successfull_update_remove_output(self):
        declaration1 = """{
  "language": "python",
  "splittable": false,
  "groups": [
    {
      "name": "channel1",
      "inputs": {
        "a": { "type": "johndoe/single_integer/1" },
        "b": { "type": "johndoe/single_integer/1" }
      },
      "outputs": {
        "sum": { "type": "johndoe/single_integer/1" },
        "diff": { "type": "johndoe/single_integer/1" }
      }
    }
  ],
  "parameters": {
  }
}"""

        code = """class Algorithm:

    def process(self, inputs, outputs):
        return True

"""

        declaration2 = """{
  "language": "python",
  "splittable": false,
  "groups": [
    {
      "name": "channel1",
      "inputs": {
        "a": { "type": "johndoe/single_integer/1" },
        "b": { "type": "johndoe/single_integer/1" }
      },
      "outputs": {
        "diff": { "type": "johndoe/single_integer/1" }
      }
    }
  ],
  "parameters": {
  }
}"""

        self.client.login(username='jackdoe', password='1234')

        response = self.client.put(self.url,
            json.dumps({
                'declaration': declaration1,
                'code': code,
            }), content_type='application/json')

        self.checkResponse(response, 204)

        response = self.client.put(self.url,
            json.dumps({
                'declaration': declaration2,
                'code': code,
            }), content_type='application/json')

        self.checkResponse(response, 204)

        algorithm = Algorithm.objects.get(author__username='jackdoe',
                                          name='personal', version=1)

        self.assertEqual(algorithm.endpoints.count(), 3)

        endpoint = algorithm.endpoints.filter(name='a')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='b')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='diff')[0]
        self.assertFalse(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')


    def test_successfull_update_change_input_to_output(self):
        declaration = """{
  "language": "python",
  "splittable": false,
  "groups": [
    {
      "name": "channel1",
      "inputs": {
        "a": { "type": "johndoe/single_integer/1" }
      },
      "outputs": {
        "sum": { "type": "johndoe/single_integer/1" },
        "b": { "type": "johndoe/single_integer/1" }
      }
    }
  ],
  "parameters": {
  }
}"""

        self.client.login(username='jackdoe', password='1234')

        response = self.client.put(self.url,
            json.dumps({
                'declaration': declaration,
            }), content_type='application/json')

        self.checkResponse(response, 204)

        algorithm = Algorithm.objects.get(author__username='jackdoe',
                                          name='personal', version=1)

        self.assertEqual(algorithm.endpoints.count(), 3)

        endpoint = algorithm.endpoints.filter(name='a')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='sum')[0]
        self.assertFalse(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='b')[0]
        self.assertFalse(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')


    def test_successfull_update_change_output_to_input(self):
        declaration1 = """{
  "language": "python",
  "splittable": false,
  "groups": [
    {
      "name": "channel1",
      "inputs": {
        "a": { "type": "johndoe/single_integer/1" },
        "b": { "type": "johndoe/single_integer/1" }
      },
      "outputs": {
        "sum": { "type": "johndoe/single_integer/1" },
        "diff": { "type": "johndoe/single_integer/1" }
      }
    }
  ],
  "parameters": {
  }
}"""

        code = """class Algorithm:

    def process(self, inputs, outputs):
        return True

"""

        declaration2 = """{
  "language": "python",
  "splittable": false,
  "groups": [
    {
      "name": "channel1",
      "inputs": {
        "a": { "type": "johndoe/single_integer/1" },
        "b": { "type": "johndoe/single_integer/1" },
        "diff": { "type": "johndoe/single_integer/1" }
      },
      "outputs": {
        "sum": { "type": "johndoe/single_integer/1" }
      }
    }
  ],
  "parameters": {
  }
}"""

        self.client.login(username='jackdoe', password='1234')

        response = self.client.put(self.url,
            json.dumps({
                'declaration': declaration1,
                'code': code,
            }), content_type='application/json')

        self.checkResponse(response, 204)

        response = self.client.put(self.url,
            json.dumps({
                'declaration': declaration2,
                'code': code,
            }), content_type='application/json')

        self.checkResponse(response, 204)

        algorithm = Algorithm.objects.get(author__username='jackdoe',
                                          name='personal', version=1)

        self.assertEqual(algorithm.endpoints.count(), 4)

        endpoint = algorithm.endpoints.filter(name='a')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='b')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='diff')[0]
        self.assertTrue(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')

        endpoint = algorithm.endpoints.filter(name='sum')[0]
        self.assertFalse(endpoint.input)
        self.assertEqual(endpoint.dataformat.fullname(), 'johndoe/single_integer/1')



class AlgorithmRetrieval(AlgorithmsAPIBase):

    def test_no_retrieval_of_confidential_algorithm_for_anonymous_user(self):
        url = reverse('api_algorithms:object', args=['johndoe', 'forked_algo'])
        response = self.client.get(url)
        self.checkResponse(response, 404)


    def test_fail_to_retrieve_with_invalid_username(self):
        self.client.login(username='johndoe', password='1234')

        url = reverse('api_algorithms:object', args=['unknown', 'forked_algo'])
        response = self.client.get(url)
        self.checkResponse(response, 404)


    def test_fail_to_retrieve_with_invalid_algorithm_name(self):
        self.client.login(username='johndoe', password='1234')

        url = reverse('api_algorithms:object', args=['johndoe', 'unknown'])
        response = self.client.get(url)
        self.checkResponse(response, 404)


    def test_no_retrieval_of_confidential_algorithm(self):
        self.client.login(username='jackdoe', password='1234')

        url = reverse('api_algorithms:object', args=['johndoe', 'forked_algo'])
        response = self.client.get(url)
        self.checkResponse(response, 404)


    def test_successful_retrieval_of_public_algorithm_for_anonymous_user(self):
        url = reverse('api_algorithms:object', args=['jackdoe', 'public_for_all'])
        response = self.client.get(url)
        data = self.checkResponse(response, 200, content_type='application/json')

        self.assertEqual(data['accessibility'], 'public')
        self.assertTrue(data['opensource'])
        self.assertEqual(data['description'], '')
        self.assertFalse(data.has_key('is_owner'))

        self.assertEqual(data['declaration'], AlgorithmsAPIBase.JSON_DECLARATION)
        self.assertEqual(data['code'], AlgorithmsAPIBase.CODE)

    def test_successful_retrieval_of_usable_algorithm_for_anonymous_user(self):
        url = reverse('api_algorithms:object', args=['jackdoe', 'usable_by_all'])
        response = self.client.get(url)
        data = self.checkResponse(response, 200, content_type='application/json')

        self.assertEqual(data['accessibility'], 'confidential')
        self.assertFalse(data['opensource'])
        self.assertEqual(data['description'], '')
        self.assertFalse(data.has_key('code'))
        self.assertFalse(data.has_key('is_owner'))


    def test_successful_retrieval_of_public_algorithm(self):
        self.client.login(username='johndoe', password='1234')

        url = reverse('api_algorithms:object', args=['jackdoe', 'public_for_all'])
        response = self.client.get(url)
        data = self.checkResponse(response, 200, content_type='application/json')

        self.assertEqual(data['is_owner'], False)
        self.assertEqual(data['accessibility'], 'public')
        self.assertTrue(data['opensource'])
        self.assertEqual(data['description'], '')

        self.assertEqual(data['declaration'], AlgorithmsAPIBase.JSON_DECLARATION)
        self.assertEqual(data['code'], AlgorithmsAPIBase.CODE)


    def test_successful_retrieval_of_usable_algorithm(self):
        self.client.login(username='johndoe', password='1234')

        url = reverse('api_algorithms:object', args=['jackdoe', 'usable_by_one_user'])
        response = self.client.get(url)
        data = self.checkResponse(response, 200, content_type='application/json')

        self.assertEqual(data['is_owner'], False)
        self.assertEqual(data['accessibility'], 'confidential')
        self.assertFalse(data['opensource'])
        self.assertEqual(data['description'], '')
        self.assertFalse(data.has_key('code'))


    def test_successful_retrieval_of_publicly_usable_algorithm(self):
        self.client.login(username='johndoe', password='1234')

        url = reverse('api_algorithms:object', args=['jackdoe', 'usable_by_all'])
        response = self.client.get(url)
        data = self.checkResponse(response, 200, content_type='application/json')

        self.assertEqual(data['is_owner'], False)
        self.assertEqual(data['accessibility'], 'confidential')
        self.assertFalse(data['opensource'])
        self.assertEqual(data['description'], '')
        self.assertFalse(data.has_key('code'))


    def test_successful_retrieval_of_confidential_algorithm(self):
        self.client.login(username='johndoe', password='1234')

        url = reverse('api_algorithms:object', args=['jackdoe', 'public_for_one_user'])
        response = self.client.get(url)
        data = self.checkResponse(response, 200, content_type='application/json')

        self.assertEqual(data['is_owner'], False)
        self.assertEqual(data['accessibility'], 'confidential')
        self.assertTrue(data['opensource'])
        self.assertEqual(data['description'], '')

        self.assertEqual(data['declaration'], AlgorithmsAPIBase.JSON_DECLARATION)
        self.assertEqual(data['code'], AlgorithmsAPIBase.CODE)


    def test_successful_retrieval_of_own_public_algorithm(self):
        self.client.login(username='jackdoe', password='1234')

        url = reverse('api_algorithms:object', args=['jackdoe', 'public_for_all'])
        response = self.client.get(url)
        data = self.checkResponse(response, 200, content_type='application/json')

        self.assertEqual(data['is_owner'], True)
        self.assertEqual(data['accessibility'], 'public')
        self.assertTrue(data['opensource'])
        self.assertEqual(data['description'], '')

        self.assertEqual(data['declaration'], AlgorithmsAPIBase.JSON_DECLARATION)
        self.assertEqual(data['code'], AlgorithmsAPIBase.CODE)

        self.assertTrue(isinstance(data['sharing'], dict))
        self.assertEqual(data['sharing']['status'], 'public')
        self.assertFalse(data['sharing'].has_key('public_users'))
        self.assertFalse(data['sharing'].has_key('usable_users'))


    def test_successful_retrieval_of_own_confidential_algorithm(self):
        self.client.login(username='jackdoe', password='1234')

        url = reverse('api_algorithms:object', args=['jackdoe', 'usable_by_one_user'])
        response = self.client.get(url)
        data = self.checkResponse(response, 200, content_type='application/json')

        self.assertEqual(data['is_owner'], True)
        self.assertEqual(data['accessibility'], 'confidential')
        self.assertFalse(data['opensource'])
        self.assertEqual(data['description'], '')

        self.assertEqual(data['declaration'], AlgorithmsAPIBase.JSON_DECLARATION)
        self.assertEqual(data['code'], AlgorithmsAPIBase.CODE)

        self.assertTrue(isinstance(data['sharing'], dict))
        self.assertEqual(data['sharing']['status'], 'shared')
        self.assertFalse(data['sharing'].has_key('shared_with'))
        self.assertEqual(data['sharing']['usable_by'], ['johndoe'])


    def test_successful_retrieval_of_own_usable_algorithm(self):
        self.client.login(username='jackdoe', password='1234')

        url = reverse('api_algorithms:object', args=['jackdoe', 'usable_by_all'])
        response = self.client.get(url)
        data = self.checkResponse(response, 200, content_type='application/json')

        self.assertEqual(data['is_owner'], True)
        self.assertEqual(data['accessibility'], 'confidential')
        self.assertFalse(data['opensource'])
        self.assertEqual(data['description'], '')

        self.assertEqual(data['declaration'], AlgorithmsAPIBase.JSON_DECLARATION)
        self.assertEqual(data['code'], AlgorithmsAPIBase.CODE)

        self.assertTrue(isinstance(data['sharing'], dict))
        self.assertEqual(data['sharing']['status'], 'usable')
        self.assertFalse(data['sharing'].has_key('shared_with'))
        self.assertFalse(data['sharing'].has_key('usable_users'))


    def test_successful_retrieval_of_own_shared_algorithm(self):
        self.client.login(username='jackdoe', password='1234')

        url = reverse('api_algorithms:object', args=['jackdoe', 'public_for_one_user'])
        response = self.client.get(url)
        data = self.checkResponse(response, 200, content_type='application/json')

        self.assertEqual(data['is_owner'], True)
        self.assertEqual(data['accessibility'], 'confidential')
        self.assertFalse(data['opensource'])
        self.assertEqual(data['description'], '')

        self.assertEqual(data['declaration'], AlgorithmsAPIBase.JSON_DECLARATION)
        self.assertEqual(data['code'], AlgorithmsAPIBase.CODE)

        self.assertTrue(isinstance(data['sharing'], dict))
        self.assertEqual(data['sharing']['status'], 'shared')
        self.assertEqual(data['sharing']['shared_with'], ['johndoe'])
        self.assertFalse(data['sharing'].has_key('usable_by'))



class AlgorithmDeletion(AlgorithmsAPIBase):

    def test_no_deletion_for_anonymous_user(self):
        url = reverse('api_algorithms:object', args=['jackdoe', 'personal', 1])
        response = self.client.delete(url)
        self.checkResponse(response, 403)


    def test_fail_to_delete_with_invalid_username(self):
        self.client.login(username='jackdoe', password='1234')

        url = reverse('api_algorithms:object', args=['unknown', 'personal', 1])
        response = self.client.delete(url)
        self.checkResponse(response, 403)


    def test_fail_to_delete_with_invalid_algorithm_name(self):
        self.client.login(username='jackdoe', password='1234')

        url = reverse('api_algorithms:object', args=['jackdoe', 'unknown', 1])
        response = self.client.delete(url)
        self.checkResponse(response, 404)


    def test_no_deletion_of_not_owned_algorithm(self):
        self.client.login(username='johndoe', password='1234')

        url = reverse('api_algorithms:object', args=['jackdoe', 'personal', 1])
        response = self.client.delete(url)
        self.checkResponse(response, 403)


    def test_successful_deletion_of_private_algorithm(self):
        self.client.login(username='jackdoe', password='1234')
        url = reverse('api_algorithms:object', args=['jackdoe', 'personal', 1])
        response = self.client.delete(url)
        self.checkResponse(response, 204)



class AlgorithmSharingAPIBase(AlgorithmsAPIBase):

    def checkJSONSharingPreferences(self, sharing_preferences, reference):
        self.assertEqual(len(sharing_preferences.keys()), len(reference.keys()))

        for (key, value) in reference.items():
            self.assertTrue(sharing_preferences.has_key(key))

            if isinstance(value, list):
                self.assertEqual(len(sharing_preferences[key]), len(value))
                for entry in value:
                    self.assertTrue(entry in sharing_preferences[key])
            else:
                self.assertEqual(sharing_preferences[key], value)


class AlgorithmSharingAPI_Failures(AlgorithmSharingAPIBase):
    def setUp(self):
        super(AlgorithmSharingAPI_Failures, self).setUp()

        self.url = reverse('api_algorithms:share', args=['jackdoe', 'personal', 1])

    def test_fail_sharing_for_anonymous_user(self):
        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
            }), content_type='application/json')

        self.checkResponse(response, 403)


    def test_fail_sharing_for_unauthorized_user(self):
        self.client.login(username='johndoe', password='1234')
        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
            }), content_type='application/json')

        self.checkResponse(response, 403)


    def test_fail_sharing_to_usable_by_unknown_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
                'users' : ['doejack'],
            }), content_type='application/json')

        data = self.checkResponse(response, 400, content_type='application/json')
        self.assertTrue(data is not None)
        self.assertTrue(isinstance(data['users'], list))


    def test_fail_sharing_to_public_for_unknown_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
                'users' : ['doejack'],
            }), content_type='application/json')

        data = self.checkResponse(response, 400, content_type='application/json')
        self.assertTrue(data is not None)
        self.assertTrue(isinstance(data['users'], list))


    def test_fail_sharing_to_unknown_status(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'notknownstatus',
            }), content_type='application/json')
        self.checkResponse(response, 400, content_type='application/json')



class NotSharedAlgorithm_SharingAPI(AlgorithmSharingAPIBase):
    def setUp(self):
        super(NotSharedAlgorithm_SharingAPI, self).setUp()

        self.url = reverse('api_algorithms:share', args=['jackdoe', 'personal', 1])

    def test_sharing_as_usable_by_all(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'usable',
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'personal', reference)


    def test_sharing_as_usable_by_one_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
                'users' : ['johndoe'],
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'usable_by': ['johndoe'],

        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'personal', reference)


    def test_sharing_as_usable_by_multiple_users(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
                'users' : ['johndoe', 'janedoe'],
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'usable_by': ['johndoe', 'janedoe'],

        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'personal', reference)


    def test_sharing_as_public_for_all(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'public',
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'personal', reference)


    def test_sharing_as_public_for_one_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
                'users' : ['johndoe'],
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'shared_with': ['johndoe'],

        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'personal', reference)


    def test_sharing_as_public_for_multiple_users(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
                'users' : ['johndoe', 'janedoe'],
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'shared_with': ['johndoe', 'janedoe'],

        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'personal', reference)



class UsableByOneUserAlgorithm_SharingAPI(AlgorithmSharingAPIBase):
    def setUp(self):
        super(UsableByOneUserAlgorithm_SharingAPI, self).setUp()

        self.url = reverse('api_algorithms:share', args=['jackdoe', 'usable_by_one_user', 1])

    def test_sharing_as_usable_by_all(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'usable',
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'usable_by_one_user', reference)


    def test_sharing_as_usable_by_the_same_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
                'users' : ['johndoe'],
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'usable_by': ['johndoe'],

        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'usable_by_one_user', reference)


    def test_sharing_as_usable_by_another_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
                'users' : ['janedoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'usable_by': ['johndoe', 'janedoe']
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'usable_by_one_user', reference)


    def test_sharing_as_usable_by_multiple_users(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
                'users' : ['janedoe', 'jacquelinedoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'usable_by': ['johndoe', 'janedoe', 'jacquelinedoe'],

        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'usable_by_one_user', reference)


    def test_sharing_as_public_for_all(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'public'
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'usable_by_one_user', reference)


    def test_sharing_as_public_for_the_same_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
                'users' : ['johndoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'shared_with': ['johndoe']
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'usable_by_one_user', reference)


    def test_sharing_as_public_for_another_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
                'users' : ['janedoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'shared_with': ['janedoe'],
            'usable_by': ['johndoe']
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'usable_by_one_user', reference)


    def test_sharing_as_public_for_multiple_users(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
                'users' : ['janedoe', 'jacquelinedoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'shared_with': ['janedoe', 'jacquelinedoe'],
            'usable_by': ['johndoe']
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'usable_by_one_user', reference)



class UsableByAllAlgorithm_SharingAPI(AlgorithmSharingAPIBase):
    def setUp(self):
        super(UsableByAllAlgorithm_SharingAPI, self).setUp()

        self.url = reverse('api_algorithms:share', args=['jackdoe', 'usable_by_all', 1])

    def test_sharing_as_usable_by_all(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable'
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'usable'
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'usable_by_all', reference)


    def test_sharing_as_usable_by_one_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
                'users': ['johndoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'usable'
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'usable_by_all', reference)


    def test_sharing_as_usable_by_multiple_users(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
                'users': ['johndoe', 'janedoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'usable'
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'usable_by_all', reference)


    def test_sharing_as_public_for_all(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public'
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'public'
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'usable_by_all', reference)


    def test_sharing_as_public_for_one_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
                'users' : ['johndoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'usable',
            'shared_with': ['johndoe']
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'usable_by_all', reference)


    def test_sharing_as_public_for_multiple_users(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
                'users' : ['johndoe', 'janedoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'usable',
            'shared_with': ['johndoe', 'janedoe']
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'usable_by_all', reference)



class PublicForOneUserAlgorithm_SharingAPI(AlgorithmSharingAPIBase):
    def setUp(self):
        super(PublicForOneUserAlgorithm_SharingAPI, self).setUp()

        self.url = reverse('api_algorithms:share', args=['jackdoe', 'public_for_one_user', 1])

    def test_sharing_as_usable_by_all(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'usable',
            'shared_with': ['johndoe']
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'public_for_one_user', reference)


    def test_sharing_as_usable_by_the_same_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
                'users' : ['johndoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'shared_with': ['johndoe']
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'public_for_one_user', reference)


    def test_sharing_as_usable_by_another_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
                'users' : ['janedoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'shared_with': ['johndoe'],
            'usable_by': ['janedoe']
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'public_for_one_user', reference)


    def test_sharing_as_usable_by_multiple_users(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
                'users' : ['janedoe', 'jacquelinedoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'shared_with': ['johndoe'],
            'usable_by': ['janedoe', 'jacquelinedoe']
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'public_for_one_user', reference)


    def test_sharing_as_public_for_all(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public'
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'public'
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'public_for_one_user', reference)


    def test_sharing_as_public_for_the_same_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
                'users' : ['johndoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'shared_with': ['johndoe']
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'public_for_one_user', reference)


    def test_sharing_as_public_for_another_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
                'users' : ['janedoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'shared_with': ['johndoe', 'janedoe']
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'public_for_one_user', reference)


    def test_sharing_as_public_for_multiple_users(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
                'users' : ['janedoe', 'jacquelinedoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'shared',
            'shared_with': ['johndoe', 'janedoe', 'jacquelinedoe']
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'public_for_one_user', reference)



class PublicForAllAlgorithm_SharingAPI(AlgorithmSharingAPIBase):
    def setUp(self):
        super(PublicForAllAlgorithm_SharingAPI, self).setUp()

        self.url = reverse('api_algorithms:share', args=['jackdoe', 'public_for_all', 1])

    def test_sharing_as_usable_by_all(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable'
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'public'
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'public_for_all', reference)


    def test_sharing_as_usable_by_one_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
                'users': ['johndoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'public'
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'public_for_all', reference)


    def test_sharing_as_usable_by_multiple_users(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'usable',
                'users': ['johndoe', 'janedoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'public'
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'public_for_all', reference)


    def test_sharing_as_public_for_all(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public'
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'public'
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'public_for_all', reference)


    def test_sharing_as_public_for_one_user(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
                'users': ['johndoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'public'
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'public_for_all', reference)


    def test_sharing_as_public_for_multiple_users(self):
        self.client.login(username='jackdoe', password='1234')

        response = self.client.post(self.url,
            json.dumps({
                'status': 'public',
                'users': ['johndoe', 'janedoe']
            }), content_type='application/json')

        data = self.checkResponse(response, 200, content_type='application/json')

        reference = {
            'status': 'public'
        }

        self.checkJSONSharingPreferences(data, reference)
        self.checkDBSharingPreferences('jackdoe', 'public_for_all', reference)
