#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################


from django import template
from django.contrib.auth.models import User


register = template.Library()


@register.inclusion_tag('backend/panels/environment_table.html', takes_context=True)
def environment_table(context, objects, id):
    '''Composes a environment list table

    This panel primarily exists for user's environment list page.

    Parameters:

        objects (iterable): An iterable containing Environment objects
        id: The HTML id to set on the generated table. This is handy for the
          filter functionality normally available on list pages.

    '''

    return dict(
            request=context['request'],
            objects=objects,
            panel_id=id,
            )


@register.inclusion_tag('backend/panels/environment_actions.html', takes_context=True)
def environment_actions(context, object, display_count):
    '''Composes the action buttons for a particular environment

    This panel primarily exists for showing action buttons for a given
    environment taking into consideration it is being displayed for a given user.

    Parameters:

        object (Environment): The environment object concerned for which the
          buttons will be drawn.
        display_count (bool): A boolean value indicating if we should or not
          display the number of queues associated to this environment as a
          button (useful for the list view)

    '''
    return dict(
            request=context['request'],
            object=object,
            display_count=display_count,
            )


@register.assignment_tag(takes_context=True)
def visible_queues(context, object):
    '''Calculates the visible queues for an environment and requestor'''
    return object.queues_for(context['request'].user)
