#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import collections
import simplejson

from django.http import Http404
from django.conf import settings
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseForbidden
from django.contrib import messages

from .models import Environment


#------------------------------------------------


@login_required
def scheduler(request):
    if not(request.user.is_superuser):
        return HttpResponseForbidden()

    from .api import scheduler as scheduler_api

    response = scheduler_api(request)

    if response.status_code != 200:
        messages.error(request, "Error contacting the scheduler at {}:{} " \
                "[{}/{}] {}".format(settings.SCHEDULER_ADDRESS,
                    settings.SCHEDULER_PORT, response.status_text,
                    response.status_code, response.data))
        return render_to_response('backend/scheduler.html', dict(),
                context_instance=RequestContext(request))

    # data for the cache plot
    cache_chart_data = [
            dict(
                label= 'Used (%d%%)' % round(100*float(response.data['cache']['size-in-megabytes'])/response.data['cache']['capacity-in-megabytes']),
                value= response.data['cache']['size-in-megabytes'],
                color= '#F7464A',
                highlight= '#FF5A5E',
                ),
            dict(
                label= 'Free (%d%%)' % round(100*float(response.data['cache']['capacity-in-megabytes'] - response.data['cache']['size-in-megabytes'])/response.data['cache']['capacity-in-megabytes']),
                value= response.data['cache']['capacity-in-megabytes'] - response.data['cache']['size-in-megabytes'],
                color= '#46BFBD',
                highlight= '#5AD3D1',
                ),
            ]

    cache_gb = int(response.data['cache']['capacity-in-megabytes'] / 1024.0)

    # Organize the data a bit
    response.data['workers'] = collections.OrderedDict(sorted([(k,v) for k,v in response.data['workers'].items()], key=lambda x: (not x[1].has_key('active'), x[1]['db_status']!='Active', x[0])))

    return render_to_response('backend/scheduler.html',
            dict(
                data=response.data,
                cache_chart_data=simplejson.dumps(cache_chart_data),
                cache_gb=cache_gb,
                ),
            context_instance=RequestContext(request))


def environment(request, name, version):
    """Displays a single database"""

    # Retrieve the data base
    environment = get_object_or_404(Environment,
        name__iexact=name, version__iexact=version)

    if not environment.accessibility_for(request.user)[0]:
        raise Http404()

    # Render the page
    return render_to_response('backend/environment.html',
                              {
                                'environment': environment,
                              },
                              context_instance=RequestContext(request))


#----------------------------------------------------------


def list_environments(request):
    """Displays all accessible (active and inactive) environments"""

    objects = Environment.objects.for_user(request.user, True).filter(active=True).order_by('-creation_date')

    # Render the page
    return render_to_response(
            'backend/environment_list.html',
            dict(
                objects=objects,
                author=request.user,
                owner=True,
                ),
            context_instance=RequestContext(request),
            )
