#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from django import template
from django.conf import settings
from ...common.texts import Messages as Texts
from ... import __version__
from collections import OrderedDict


register = template.Library()


#--------------------------------------------------


@register.inclusion_tag('ui/bar.html', takes_context=True)
def navbar(context):
    request = context['request']
    return {
        'request': context['request'],
        'public_urls': [
          ('experiments', 'experiments:public-list'),
          (False, ''),
          ('toolchains', 'toolchains:public-list'),
          ('algorithms', 'algorithms:public-list'),
          ('libraries', 'libraries:public-list'),
          ('dataformats', 'dataformats:public-list'),
          (False, ''),
          ('attestations', 'attestations:public-list'),
          ('reports', 'reports:public-list'),
          ('searches', 'search:public-list'),
          ('teams', 'teams:public-list'),
          (False, ''),
          ('databases', 'databases:list'),
          ('environments', 'backend:list-environments'),
          ],
        'user_urls': [
          ('experiments', 'experiments:list', True),
          (False, '', False),
          ('toolchains', 'toolchains:list', True),
          ('algorithms', 'algorithms:list', True),
          ('libraries', 'libraries:list', True),
          ('dataformats', 'dataformats:list', True),
          (False, '', False),
          ('attestations', 'attestations:list', True),
          ('reports', 'reports:list', True),
          ('searches', 'search:list', True),
          ('teams', 'teams:list', True),
          (False, '', False),
          ('databases', 'databases:list', False),
          ('environments', 'backend:list-environments', False),
          ],
        }


#--------------------------------------------------


@register.inclusion_tag('ui/contribution_breadcrumb.html', takes_context=True)
def contribution_breadcrumb(context, obj):
    name_plural = obj.get_verbose_name_plural()
    return {
            'request': context['request'],
            'object': obj,
            'name_plural': name_plural,
            'listurl': name_plural + ':list',
            'public_listurl': name_plural + ':public-list',
            'viewurl': name_plural + ':view-latest',
           }


#--------------------------------------------------


@register.inclusion_tag('ui/filter_script.html')
def filter_script(panel_id, text_filter_id, privacy_filter_id):
    return dict(
            panel_id=panel_id,
            text_filter_id=text_filter_id,
            privacy_filter_id=privacy_filter_id,
            )


#--------------------------------------------------


@register.inclusion_tag('ui/smart_selector.html')
def smart_selector(id):
    return { 'panel_id': id,
           }


#--------------------------------------------------


@register.inclusion_tag('ui/tabs.html', takes_context=True)
def list_tabs(context, user, tab):
    name = 'list' if not user.is_anonymous() else 'public-list'
    return dict(
            request=context['request'],
            user=user,
            tab=tab,

            #tab name -> list url mappings
            user_tabs=OrderedDict([
              ('experiments', 'experiments:' + name),
              (False, ''),
              ('toolchains', 'toolchains:' + name),
              ('algorithms', 'algorithms:' + name),
              ('libraries', 'libraries:' + name),
              ('dataformats', 'dataformats:' + name),
              (None, ''), #note it has to be different as this is a dictionary!
              ('attestations', 'attestations:' + name),
              ('reports', 'reports:' + name),
              ('searches', 'search:' + name),
              ('teams', 'teams:' + name),
              ]),
            system_tabs=OrderedDict([
              ('databases', 'databases:list'),
              ('environments', 'backend:list-environments'),
              ]),
            )


#--------------------------------------------------


@register.inclusion_tag('ui/list_selector.html')
def list_selector(id):
    return { 'panel_id': id,
           }


#--------------------------------------------------


@register.inclusion_tag('ui/multiple_selector.html')
def multiple_selector(id):
    return { 'panel_id': id,
           }


#--------------------------------------------------


@register.inclusion_tag('ui/doc_editor.html', takes_context=True)
def doc_editor(context, obj, url_name, editable=True):
    return {
             'request': context['request'],
             'owner': obj.author == context['request'].user,
             'object': obj,
             'url_name': url_name,
             'texts': Texts,
             'editable': editable,
           }


#--------------------------------------------------


@register.inclusion_tag('ui/history.html', takes_context=True)
def history(context, plural, obj, panel_id, height):
    return {
             'plural': plural,
             'object': obj,
             'history': obj.json_history(context['request'].user),
             'panel_id': panel_id,
             'height': height,
             'URL_PREFIX': context['URL_PREFIX'],
           }


#--------------------------------------------------


@register.simple_tag
def code_editor_scripts(modes):
    js = [
        'lib/codemirror.js',
        'addon/fold/brace-fold.js',
        'addon/fold/foldcode.js',
        'addon/fold/foldgutter.js',
        'addon/fold/indent-fold.js',
        'addon/mode/overlay.js',
        'addon/search/search.js',
        'addon/search/searchcursor.js',
        'addon/dialog/dialog.js',
        'addon/display/rulers.js',
        ]

    # adds mode-related CodeMirror plugins
    js += ['mode/%s/%s.js' % (k,k) for k in modes.split(',')]

    # sets include path
    js = ['%s/%s' % (settings.CODEMIRROR_PATH, k) for k in js]

    # inserts the CodeMirror editor defaults
    js.append("ui/js/codemirror-defaults.js")

    js = ['<script src="%s%s?v%s" type="text/javascript" charset="utf-8"></script>' % (settings.STATIC_URL, k, __version__) for k in js]

    return '\n'.join(js)


#--------------------------------------------------


@register.simple_tag
def code_editor_css():
    css = [
        '%s/lib/codemirror.css' % settings.CODEMIRROR_PATH,
        '%s/addon/fold/foldgutter.css' % settings.CODEMIRROR_PATH,
        'ui/css/codemirror-overrides.css',
    ]

    if settings.CODEMIRROR_THEME != 'default':
        css.append('%s/theme/%s.css' % (settings.CODEMIRROR_PATH, settings.CODEMIRROR_THEME))

    return '\n'.join(map(lambda x: '<link rel="stylesheet" href="%s%s?v%s" type="text/css" media="screen" />' % \
                            (settings.STATIC_URL, x, __version__), css))


#--------------------------------------------------


@register.inclusion_tag('ui/save_as_dialog.html')
def save_as_dialog(id):
    return { 'dialog_id': id,
             'URL_PREFIX': settings.URL_PREFIX,
           }


#--------------------------------------------------


@register.inclusion_tag('ui/messages.html')
def messages(request):
    """Sets-up the HTML for message display"""
    from django.contrib import messages as django_messages
    return dict(
            messages=django_messages.get_messages(request),
            DEFAULT_MESSAGE_LEVELS=django_messages.DEFAULT_LEVELS,
            )


#--------------------------------------------------


@register.filter
def join_by(the_list, attr_or_method):
    """Joins the list attributes or result of methods"""

    def _getval(o, n):
        attr = getattr(o, n)
        if hasattr(attr, '__call__'): return attr()
        else: return attr

    return ", ".join(unicode(_getval(i, attr_or_method)) for i in the_list)


#--------------------------------------------------


@register.filter
def getkey(mapping, key):
    """Returns the given dictionary key (useful in templates)"""
    return mapping.get(key, '')


#--------------------------------------------------


@register.filter
def split_fullname(s):
    return s.split('/')
