#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################


import logging
logger = logging.getLogger(__name__)

from django.core.management.base import BaseCommand, CommandError

from ....utils import scheduler
from ....backend.models import Queue


class Command(BaseCommand):

    help = 'Re-configures the scheduler'

    def handle(self, *ignored, **arguments):

        # Setup this command's logging level
        global logger
        arguments['verbosity'] = int(arguments['verbosity'])
        if arguments['verbosity'] >= 1:
            if arguments['verbosity'] == 1: logger.setLevel(logging.INFO)
            elif arguments['verbosity'] >= 2: logger.setLevel(logging.DEBUG)

        configuration = {}
        for queue in Queue.objects.all():
            configuration[queue.name] = queue.as_json()

        # Send the configuration to the Scheduler
        status_and_data = scheduler.putMessage('/queue-configuration',
                data=configuration)

        if status_and_data is None:
            logger.error('Could *not* connect to the scheduler')

        status, data = status_and_data
        if status != 200:
            logger.error('Scheduler answered with *error* status %d:', status)
            logger.error(data + '\n')
        else:
            logger.info('Scheduler reconfiguration succesfully launched')
